/* Copyright (c) 1997-2013
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Darmstadt, Germany)
   http://www.polymake.org

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
--------------------------------------------------------------------------------
   $Project: polymake $$Id: arithmetic.h 11253 2013-05-16 07:52:51Z joswig $
*/

/** @file TropicalArithmetic.h
    @brief Implementation of classes relevant for tropical computations.
*/

#ifndef POLYMAKE_TROPICAL_ARITHMETIC_H
#define POLYMAKE_TROPICAL_ARITHMETIC_H

#include "polymake/Polynomial.h"
#include "polymake/Rational.h"
#include "polymake/Matrix.h"
#include "polymake/Vector.h"

namespace polymake { namespace tropical {

struct Max;

struct Min {
   template <typename T>
   static const T& apply(const T& x, const T& y) { return x<y ? x : y; }

   static const int orientation() { return 1; }

   typedef Max dual;
};

struct Max {
   template <typename T>
   static const T& apply(const T& x, const T& y) { return x>y ? x : y; }

   static int orientation() { return -1; }

   typedef Min dual;
};


typedef std::pair<Matrix<int>, Vector<Rational> > TropicalPolynomial;

typedef Ring<UniPolynomial<Rational,Rational>,int> PuiseuxRing;

typedef Polynomial<UniPolynomial<Rational,Rational>,int> PuiseuxPolynomial;

} }

#endif // POLYMAKE_TROPICAL_ARITHMETIC_H

// Local Variables:
// mode:C++
// c-basic-offset:3
// indent-tabs-mode:nil
// End:
